==> open-xchange-file-storage-onedrive_7.8.4.orig.tar.bz2: open-xchange-file-storage-onedrive-7.8.4/com.openexchange.file.storage.onedrive/src/com/openexchange/file/storage/onedrive/osgi/OneDriveServiceRegisterer.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.file.storage.onedrive.osgi;

import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;
import org.osgi.framework.ServiceRegistration;
import org.osgi.util.tracker.ServiceTrackerCustomizer;
import com.openexchange.file.storage.CompositeFileStorageAccountManagerProvider;
import com.openexchange.file.storage.FileStorageAccountManagerProvider;
import com.openexchange.file.storage.FileStorageService;
import com.openexchange.file.storage.onedrive.OneDriveConstants;
import com.openexchange.file.storage.onedrive.OneDriveFileStorageService;
import com.openexchange.file.storage.onedrive.oauth.OneDriveOAuthAccountAssociationProvider;
import com.openexchange.oauth.OAuthAccountDeleteListener;
import com.openexchange.oauth.association.spi.OAuthAccountAssociationProvider;

/**
 * {@link OneDriveServiceRegisterer}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class OneDriveServiceRegisterer implements ServiceTrackerCustomizer<FileStorageAccountManagerProvider, FileStorageAccountManagerProvider> {

    private final BundleContext context;
    private FileStorageAccountManagerProvider provider;
    private OneDriveFileStorageService service;
    private ServiceRegistration<FileStorageService> serviceRegistration;
    private ServiceRegistration<OAuthAccountDeleteListener> listenerRegistration;
    private ServiceRegistration<OAuthAccountAssociationProvider> associationProviderRegistration; // guarded by synchronized

    /**
     * Initializes a new {@link OneDriveServiceRegisterer}.
     */
    public OneDriveServiceRegisterer(final BundleContext context) {
        super();
        this.context = context;
    }

    /** For testing only */
    public OneDriveFileStorageService getService() {
        return service;
    }

    @Override
    public FileStorageAccountManagerProvider addingService(final ServiceReference<FileStorageAccountManagerProvider> reference) {
        final FileStorageAccountManagerProvider provider = context.getService(reference);
        if (!provider.supports(OneDriveConstants.ID)) {
            context.ungetService(reference);
            return null;
        }
        synchronized (this) {
            OneDriveFileStorageService service = this.service;
            if (null == service) {
                /*
                 * Try to create Microsoft OneDrive service
                 */
                service = new OneDriveFileStorageService(Services.getServices());
                this.serviceRegistration = context.registerService(FileStorageService.class, service, null);
                this.listenerRegistration = context.registerService(OAuthAccountDeleteListener.class, service, null);
                this.associationProviderRegistration = context.registerService(OAuthAccountAssociationProvider.class, new OneDriveOAuthAccountAssociationProvider(service), null);
                this.service = service;
                this.provider = provider;
            } else {
                /*
                 * Already created before, but new provider
                 */
                CompositeFileStorageAccountManagerProvider compositeProvider = service.getCompositeAccountManager();
                if (null == compositeProvider) {
                    compositeProvider = new CompositeFileStorageAccountManagerProvider();
                    compositeProvider.addProvider(this.provider);
                    unregisterService(null);
                    service = new OneDriveFileStorageService(Services.getServices(), compositeProvider);
                    this.serviceRegistration = context.registerService(FileStorageService.class, service, null);
                    this.listenerRegistration = context.registerService(OAuthAccountDeleteListener.class, service, null);
                    this.associationProviderRegistration = context.registerService(OAuthAccountAssociationProvider.class, new OneDriveOAuthAccountAssociationProvider(service), null);
                    this.service = service;
                    this.provider = compositeProvider;
                }
                compositeProvider.addProvider(provider);
            }
        }
        return provider;
    }

    @Override
    public void modifiedService(final ServiceReference<FileStorageAccountManagerProvider> reference, final FileStorageAccountManagerProvider provider) {
        // Ignore
    }

    @Override
    public void removedService(final ServiceReference<FileStorageAccountManagerProvider> reference, final FileStorageAccountManagerProvider provider) {
        if (null != provider) {
            synchronized (this) {
                final CompositeFileStorageAccountManagerProvider compositeProvider = this.service.getCompositeAccountManager();
                if (null == compositeProvider) {
                    unregisterService(reference);
                } else {
                    compositeProvider.removeProvider(provider);
                    if (!compositeProvider.hasAnyProvider()) {
                        unregisterService(reference);
                    }
                }
            }
        }
    }

    private void unregisterService(final ServiceReference<FileStorageAccountManagerProvider> ref) {
        ServiceRegistration<FileStorageService> serviceRegistration = this.serviceRegistration;
        if (null != serviceRegistration) {
            this.serviceRegistration = null;
            serviceRegistration.unregister();
        }

        ServiceRegistration<OAuthAccountDeleteListener> listenerRegistration = this.listenerRegistration;
        if (null != listenerRegistration) {
            this.listenerRegistration = null;
            listenerRegistration.unregister();
        }

        ServiceRegistration<OAuthAccountAssociationProvider> associationProviderRegistration = this.associationProviderRegistration;
        if (null != associationProviderRegistration) {
            this.associationProviderRegistration = null;
            associationProviderRegistration.unregister();
        }

        ServiceReference<FileStorageAccountManagerProvider> reference = ref;
        if (null != reference) {
            context.ungetService(reference);
        }

        this.service = null;
    }

}

==> open-xchange-file-storage-onedrive_7.10.2.orig.tar.bz2: open-xchange-file-storage-onedrive-7.10.2/com.openexchange.file.storage.onedrive/src/com/openexchange/file/storage/onedrive/osgi/OneDriveServiceRegisterer.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.file.storage.onedrive.osgi;

import org.osgi.framework.BundleContext;
import com.openexchange.file.storage.CompositeFileStorageAccountManagerProvider;
import com.openexchange.file.storage.oauth.AbstractOAuthFileStorageService;
import com.openexchange.file.storage.oauth.osgi.AbstractCloudStorageServiceRegisterer;
import com.openexchange.file.storage.onedrive.OneDriveConstants;
import com.openexchange.file.storage.onedrive.OneDriveFileStorageService;
import com.openexchange.file.storage.onedrive.oauth.OneDriveOAuthAccountAssociationProvider;
import com.openexchange.oauth.association.spi.OAuthAccountAssociationProvider;
import com.openexchange.server.ServiceLookup;

/**
 * {@link OneDriveServiceRegisterer}
 *
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * @author <a href="mailto:ioannis.chouklis@open-xchange.com">Ioannis Chouklis</a>
 */
public final class OneDriveServiceRegisterer extends AbstractCloudStorageServiceRegisterer {

    /**
     * Initializes a new {@link OneDriveServiceRegisterer}.
     */
    public OneDriveServiceRegisterer(BundleContext context, ServiceLookup services) {
        super(context, services);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.openexchange.file.storage.oauth.osgi.AbstractCloudStorageServiceRegisterer#getCloudFileStorageService(com.openexchange.server.ServiceLookup)
     */
    @Override
    protected AbstractOAuthFileStorageService getCloudFileStorageService() {
        return new OneDriveFileStorageService(getServiceLookup());
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.openexchange.file.storage.oauth.osgi.AbstractCloudStorageServiceRegisterer#getCloudFileStorageService(com.openexchange.server.ServiceLookup, com.openexchange.file.storage.CompositeFileStorageAccountManagerProvider)
     */
    @Override
    protected AbstractOAuthFileStorageService getCloudFileStorageService(CompositeFileStorageAccountManagerProvider compositeProvider) {
        return new OneDriveFileStorageService(getServiceLookup(), compositeProvider);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.openexchange.file.storage.oauth.osgi.AbstractCloudStorageServiceRegisterer#getOAuthAccountAssociationProvider(com.openexchange.file.storage.oauth.AbstractOAuthFileStorageService)
     */
    @Override
    protected OAuthAccountAssociationProvider getOAuthAccountAssociationProvider(AbstractOAuthFileStorageService storageService) {
        return new OneDriveOAuthAccountAssociationProvider((OneDriveFileStorageService) storageService);
    }

    /*
     * (non-Javadoc)
     * 
     * @see com.openexchange.file.storage.oauth.osgi.AbstractCloudStorageServiceRegisterer#getProviderId()
     */
    @Override
    protected String getProviderId() {
        return OneDriveConstants.ID;
    }
}
